@@ -1,4 +1,4 @@
-# Copyright (c) 2014 Riverbed Technology, Inc.
+# Copyright (c) 2019 Riverbed Technology, Inc.
 #
 # This software is licensed under the terms and conditions of the MIT License
 # accompanying the software ("License").  This software is distributed "AS IS"
@@ -81,7 +81,7 @@ class HistoryManager(object):
         try:
             readline.clear_history()
             if self.current.obj:
-                with open(self.current.filename, 'r') as f:
+                with open(self.current.filename, 'rb') as f:
                     lines = pickle.load(f)
 
                 for line in lines:
@@ -99,11 +99,11 @@ class HistoryManager(object):
 
         if self.current.obj:
             lines = []
-            for i in range(readline.get_current_history_length()+1):
+            for i in range(1, readline.get_current_history_length()+1):
                 line = readline.get_history_item(i)
                 if line:
                     lines.append(str(line))
-            with open(self.current.filename, 'w') as f:
+            with open(self.current.filename, 'wb') as f:
                 pickle.dump(lines, f)
         else:
             readline.write_history_file(self.current.filename)
@@ -177,7 +177,7 @@ Add custom headers as <header>:<value>"""
 
     def main(self):
         if self.rest.conn is None:
-            print "Not connected - use 'connect HOST'"
+            print("Not connected - use 'connect HOST'")
             return
 
         self.rest.history.set_context('body')
@@ -195,10 +195,10 @@ Add custom headers as <header>:<value>"""
                     try:
                         line = self.rest.raw_input_no_history("")
                     except EOFError:
-                        print ""
+                        print("")
                         break
                     except KeyboardInterrupt:
-                        print "(aborted)"
+                        print("(aborted)")
                         return
 
                     if line == '.':
@@ -221,15 +221,14 @@ Add custom headers as <header>:<value>"""
             else:
                 body = None
 
-            if self.args > len(self.positional_args):
+            if self.args:
                 params = {}
                 headers = {}
 
                 for arg in self.args:
                     m = re.match('[^:=]*([:=])', arg)
                     if not m or m.group(1) not in ':=':
-                        raise ('Invalid argument, expected either k=v for '
-                               'URL param, or k:v for header')
+                        raise ValueError('Invalid argument, expected either k=v for ')
 
                     (k, v) = arg.split(m.group(1), 1)
                     if m.group(1) == ':':
@@ -240,7 +239,7 @@ Add custom headers as <header>:<value>"""
                 headers = None
                 params = None
 
-            print "Issuing %s" % self.keyword
+            print("Issuing %s" % self.keyword)
             outputlines = None
             if self.rest.jsonmode:
                 (data, resp) = self.rest.conn.json_request(
@@ -263,8 +262,8 @@ Add custom headers as <header>:<value>"""
                     else:
                         outputlines = resp.text
 
-            print "HTTP Status %s: %s bytes" % (resp.status_code,
-                                                len(resp.content))
+            print("HTTP Status %s: %s bytes" % (resp.status_code,
+                                                len(resp.content)))
             if outputlines is not None:
                 count = len(outputlines.split('\n'))
                 try:
@@ -298,7 +297,7 @@ class POST(Method):
     help = """Perform an HTTP POST.
 
 You will be prompted to enter the request body.  Use 'mode' to
-change the defulat request body format as 'json' or 'text'.
+change the default request body format as 'json' or 'text'.
 
 """ + Method.help
     readbody = True
@@ -309,7 +308,7 @@ class PUT(Method):
     help = """Perform an HTTP PUT.
 
 You will be prompted to enter the request body.  Use 'mode' to
-change the defulat request body format as 'json' or 'text'.
+change the default request body format as 'json' or 'text'.
 
 """ + Method.help
     readbody = True
@@ -320,7 +319,7 @@ class Help(RestCommand):
     help = 'Verbose instructions'
 
     def main(self):
-        print """
+        print("""
 This shell provides an interactive session for issuing REST calls
 to a remote server.
 
@@ -335,11 +334,11 @@ For HTTP methods that require a body, you will be prompted to enter
 the body on a separate line.
 
 Available commands:
-"""
+""")
         w = max([len(sub.keyword) for sub in self.rest.cmd.subcommands])
         for sub in self.rest.cmd.subcommands:
-            print "  %-*s  %s" % (w, sub.keyword, sub.help.split('\n')[0])
-        print ""
+            print("  %-*s  %s" % (w, sub.keyword, sub.help.split('\n')[0]))
+        print("")
 
 
 class Mode(RestCommand):
@@ -364,9 +363,6 @@ class Command(Application):
         self.interactive = True
         self.jsonmode = True
 
-    def parse_args(self):
-        super(Command, self).parse_args()
-
     def add_positional_args(self):
         super(Command, self).add_positional_args()
         # self.add_positional_arg('host', 'Rest hostname or IP address')
@@ -389,10 +385,9 @@ class Command(Application):
         if self.filelines:
             return self.filelines.pop(0)
         else:
-            input = raw_input(prompt)
-            readline.remove_history_item(
-                readline.get_current_history_length() - 1
-            )
+            input = input(prompt)
+            readline.clear_history()
+
             return input
 
     def main(self):
@@ -422,24 +417,23 @@ class Command(Application):
         if self.options.file:
             with open(self.options.file, 'r') as f:
                 self.filelines = [line.rstrip() for line in f]
-
-        print "REST Shell ('help' or 'quit' when done)"
-        print "Current mode is 'json', use 'mode text' to switch to raw text"
+        print("REST Shell ('help' or 'quit' when done)")
+        print("Current mode is 'json', use 'mode text' to switch to raw text")
         while True:
             if self.filelines:
                 line = self.filelines.pop(0)
                 self.interactive = False
-                print "Command: %s" % line
+                print("Command: %s" % line)
             else:
                 try:
                     if self.conn is not None:
                         prompt = "%s> " % self.conn.hostname
                     else:
                         prompt = "> "
-                    line = raw_input(prompt)
+                    line = input(prompt)
                     self.interactive = True
                 except EOFError:
-                    print ""
+                    print("")
                     break
 
             if not line:
@@ -456,6 +450,6 @@ class Command(Application):
             except SystemExit as e:
                 pass
             except Exception as e:
-                print ('Command raised an error, see log for traceback:\n%s\n'
-                       % str(e))
+                print(('Command raised an error, see log for traceback:\n%s\n'
+                       % str(e)))
                 logger.exception("Command raised an error")
