@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# Copyright (c) 2014 Riverbed Technology, Inc.
+# Copyright (c) 2019 Riverbed Technology, Inc.
 #
 # This software is licensed under the terms and conditions of the MIT License
 # accompanying the software ("License").  This software is distributed "AS IS"
@@ -35,10 +35,13 @@ except ImportError:
 
 
 import logging
+
 if __name__ == '__main__':
     logger = logging.getLogger('steel')
+    logger.setLevel(logging.DEBUG)
 else:
     logger = logging.getLogger(__name__)
+    logger.setLevel(logging.DEBUG)
 
 try:
     __VERSION__ = get_distribution('steelscript').version
@@ -58,12 +61,17 @@ LOGFILE = None
 
 STEELSCRIPT_CORE = ['steelscript',
                     'steelscript.netprofiler',
-                    'steelscript.netshark']
+                    'steelscript.appresponse',
+                    'steelscript.packets',
+                    'steelscript.wireshark',
+                    ]
 
 STEELSCRIPT_APPFW = ['steelscript.appfwk',
-                     'steelscript.wireshark',
                      'steelscript.appfwk.business-hours']
 
+STEELSCRIPT_STEELHEAD = ['steelscript.cmdline',
+                         'steelscript.scc',
+                         'steelscript.steelhead']
 
 logging_initialized = False
 
@@ -301,7 +309,7 @@ class BaseCommand(object):
         try:
             self.main()
         except MainFailed as e:
-            console(e.message, logging.ERROR)
+            console(e, logging.ERROR)
             sys.exit(1)
 
     def validate_args(self):
@@ -316,7 +324,6 @@ class BaseCommand(object):
           super(<subclass>, self).setup()
 
         """
-
         pass
 
     def setup(self):
@@ -328,7 +335,7 @@ class BaseCommand(object):
           super(<subclass>, self).setup()
 
         This will ensure the any setup required of the parent
-        classes is perfomed as well.
+        classes is performed as well.
 
         """
         pass
@@ -389,16 +396,16 @@ class InstallCommand(BaseCommand):
 
         group.add_option(
             '-d', '--dir', action='store', default=None,
-            help='Directory to use for installation')
+            help='Directory to use for offline installation')
 
         group.add_option(
-            # Install packages from gitlab
+            # Install packages from GitHub
             '-g', '--github', action='store_true',
             help="Install packages from github")
 
         group.add_option(
-            # Install packages from gitlab
-            '-G', '--gitlab', action='store_true',
+            # Dev only - install packages from bitbucket
+            '-B', '--bitbucket', action='store_true',
             help=optparse.SUPPRESS_HELP)
 
         group.add_option(
@@ -408,13 +415,12 @@ class InstallCommand(BaseCommand):
 
         group.add_option(
             '--develop', action='store_true',
-            help='Combine with --gitlab to checkout packages')
+            help='Combine with --github to checkout packages in develop mode')
 
         group.add_option(
             '-p', '--package', action='append', dest='packages',
             help='Package to install (may specify more than once)')
 
-        # Install packages from gitlab
         group.add_option(
             '--appfwk', action='store_true',
             help='Install all application framework packages')
@@ -423,6 +429,10 @@ class InstallCommand(BaseCommand):
             '--pip-options', default='',
             help='Additional options to pass to pip')
 
+        group.add_option(
+            '--steelhead', action='store_true',
+            help='Install optional steelhead packages')
+
         parser.add_option_group(group)
 
     def validate_args(self):
@@ -430,6 +440,8 @@ class InstallCommand(BaseCommand):
             self.options.packages = STEELSCRIPT_CORE
             if self.options.appfwk:
                 self.options.packages.extend(STEELSCRIPT_APPFW)
+            if self.options.steelhead:
+                self.options.packages.extend(STEELSCRIPT_STEELHEAD)
 
         if self.options.develop:
             if not self.options.dir:
@@ -457,15 +469,15 @@ class InstallCommand(BaseCommand):
                             '$ sudo pip install virtualenv\n\n'
                             'Create a new virtual environment\n'
                             '$ virtualenv <name>\n\n'
-                            'Activate the new virtal environment\n'
+                            'Activate the new virtual environment\n'
                             '$ source <name>/bin/activate\n\n')
                 sys.exit(1)
 
         if self.options.giturl:
             self.install_git(self.options.giturl)
 
-        elif self.options.gitlab:
-            self.install_gitlab()
+        elif self.options.bitbucket:
+            self.install_bitbucket()
 
         elif self.options.github:
             self.install_github()
@@ -477,22 +489,17 @@ class InstallCommand(BaseCommand):
             self.install_pip()
 
     def prepare_appfwk(self):
-        # Manually install django-admin-tools because it will
-        # die with recent versions of pip
-        shell(cmd=(('pip install {pip_options} '
-                    '--allow-unverified django-admin-tools '
-                    'django-admin-tools==0.5.1')
-                   .format(pip_options=self.options.pip_options)),
-              msg='Installing django-admin-tools')
+        # Any special-case appfwk packages should be handled here.
 
+        # Check for numpy/pandas and check for prerequisites
         if not all([pkg_installed('numpy'), pkg_installed('pandas')]):
             import platform
             if platform.system() == 'Windows':
-                console('Please install the packages `numpy` and `pandas`\n'
-                        'manually using the instructions found here:\n'
-                        'https://support.riverbed.com/apis/steelscript/'
-                        'appfwk/install.html#detailed-installation\n')
-                sys.exit(1)
+                if not check_vcpython27():
+                    console('Please follow the instructions found here:\n'
+                            'https://support.riverbed.com/apis/steelscript/'
+                            'appfwk/install.html#detailed-installation\n')
+                    sys.exit(1)
             elif not exe_installed('gcc'):
                 console('Unable to detect installed compiler `gcc`\n'
                         'which is required for installation of\n'
@@ -522,9 +529,25 @@ class InstallCommand(BaseCommand):
                             'product-lines/steelscript\n')
                 sys.exit(1)
 
+    def prepare_cmdline(self):
+        # Manually install pycrypto if it is a windows platform
+        if not pkg_installed('pycrypto'):
+            import platform
+            if platform.system() == 'Windows' and not check_vcpython27():
+                console('Please follow the instructions found here:\n'
+                        'https://support.riverbed.com/apis/steelscript/'
+                        'install/steelhead.html#detailed-installation')
+                sys.exit(1)
+
     def install_git(self, baseurl):
         """Install packages from a git repository."""
-        check_git()
+        try:
+            check_git()
+        except ShellFailed:
+            console('no\ngit is not installed, please install git to continue',
+                    lvl=logging.ERROR)
+            sys.exit(1)
+
         check_install_pip()
         for pkg in self.options.packages:
             if pkg_installed(pkg) and not self.options.upgrade:
@@ -536,6 +559,9 @@ class InstallCommand(BaseCommand):
             if pkg == 'steelscript.appfwk':
                 self.prepare_appfwk()
 
+            if pkg == 'steelscript.cmdline':
+                self.prepare_cmdline()
+
             if self.options.develop:
                 # Clone the git repo
                 outdir = os.path.join(self.options.dir, pkg.replace('.', '-'))
@@ -554,21 +580,17 @@ class InstallCommand(BaseCommand):
                     )
 
                 # Now install this git repo in develop mode
-                shell(cmd=('cd {outdir}; python setup.py develop'
+                shell(cmd=('cd {outdir}; pip install -e .'
                            .format(outdir=outdir)),
                       msg=('Installing {pkg}'.format(pkg=pkg)))
             else:
-                shell(cmd=('pip install {pip_options} {upgrade}git+{repo}'
-                           .format(repo=repo,
-                                   upgrade=('-U --no-deps '
-                                            if self.options.upgrade else ''),
-                                   pip_options=self.options.pip_options)),
-                      msg=('Installing {pkg}'.format(pkg=pkg)))
+                suffix = 'git+{repo} '.format(repo=repo)
+                self.pip_install_pkg_with_upgrade(pkg, suffix=suffix)
 
-    def install_gitlab(self):
-        """Install packages from gitlab internal to riverbed."""
+    def install_bitbucket(self):
+        """Install packages from bitbucket internal to riverbed."""
         check_install_pip()
-        self.install_git('https://gitlab.lab.nbttech.com/steelscript')
+        self.install_git('https://code.rvbdtechlabs.net/scm/sct')
 
     def install_github(self):
         """Install packages from github.com/riverbed."""
@@ -589,15 +611,32 @@ class InstallCommand(BaseCommand):
             if pkg == 'steelscript.appfwk':
                 self.prepare_appfwk()
 
-            cmd = (('pip install {pip_options} {upgrade}--no-index '
-                    '--find-links=file://{dir} {pkg}')
-                   .format(dir=self.options.dir, pkg=pkg,
-                           upgrade=('-U --no-deps'
-                                    if self.options.upgrade else ''),
-                           pip_options=self.options.pip_options))
-            shell(cmd=cmd,
-                  msg=('Installing {pkg}'
-                       .format(pkg=pkg, dir=self.options.dir)))
+            if pkg == 'steelscript.cmdline':
+                self.prepare_cmdline()
+
+            suffix = ('--no-index --find-links=file://{dir} {pkg}'.
+                      format(dir=self.options.dir, pkg=pkg))
+            self.pip_install_pkg_with_upgrade(pkg, suffix=suffix)
+
+    def pip_install_pkg_with_upgrade(self, pkg, suffix=''):
+        """Perform "Only if needed" recursive upgrade install for pkg and
+        its dependencies.
+
+        https://pip.pypa.io/en/latest/user_guide.html
+        #only-if-needed-recursive-upgrade
+        """
+        if self.options.upgrade:
+            self.pip_install_pkg(pkg, upgrade=True, suffix=suffix)
+        self.pip_install_pkg(pkg, suffix=suffix)
+
+    def pip_install_pkg(self, pkg, upgrade=False, suffix=''):
+        cmd = (('pip install {pip_options} {upgrade} {suffix}')
+               .format(suffix=suffix,
+                       upgrade=('-U --no-deps'
+                                if upgrade else ''),
+                       pip_options=self.options.pip_options))
+        shell(cmd=cmd,
+              msg=('Installing {pkg}'.format(pkg=pkg)))
 
     def install_pip(self):
         check_install_pip()
@@ -610,14 +649,10 @@ class InstallCommand(BaseCommand):
             if pkg == 'steelscript.appfwk':
                 self.prepare_appfwk()
 
-            cmd = ('pip install {pip_options} {upgrade} {pkg}'
-                   .format(pkg=pkg,
-                           upgrade=('-U --no-deps'
-                                    if self.options.upgrade else ''),
-                           pip_options=self.options.pip_options))
-            shell(cmd=cmd,
-                  msg=('Installing {pkg}'
-                       .format(pkg=pkg, dir=self.options.dir)))
+            if pkg == 'steelscript.cmdline':
+                self.prepare_cmdline()
+
+            self.pip_install_pkg_with_upgrade(pkg, suffix=pkg)
 
 
 class UninstallCommand(BaseCommand):
@@ -714,17 +749,17 @@ def prompt(msg, choices=None, default=None, password=False):
         if password:
             value = getpass.getpass(msg)
         else:
-            value = raw_input(msg)
+            value = input(msg)
 
         if not value:
             if default is not None:
                 value = default
             else:
-                print 'Please enter a valid response.'
+                print('Please enter a valid response.')
 
         if choices and value not in choices:
-            print ('Please choose from the following choices (%s)' %
-                   '/'.join(choices))
+            print(('Please choose from the following choices (%s)' %
+                   '/'.join(choices)))
             value = None
 
     return value
@@ -734,7 +769,7 @@ def add_log_options(parser):
     group = optparse.OptionGroup(parser, "Logging Parameters")
     group.add_option("--loglevel",
                      help="log level: debug, warn, info, critical, error",
-                     choices=LOG_LEVELS.keys(), default="info")
+                     choices=list(LOG_LEVELS.keys()), default="info")
     group.add_option("--logfile",
                      help="log file, use '-' for stdout", default=None)
     parser.add_option_group(group)
@@ -825,13 +860,16 @@ def shell(cmd, msg=None, allow_fail=False, exit_on_fail=True,
         console(msg + '...', newline=False)
 
     def enqueue_output(out, queue):
-        for line in iter(out.readline, b''):
+        for line in iter(out.readline, ''):
+            logger.debug('putting {} on queue'.format(line))
             queue.put(line)
         out.close()
+        logger.debug('closing queue')
 
     logger.info('Running command: %s' % cmd)
     proc = subprocess.Popen(cmd, shell=True, env=env, cwd=cwd, bufsize=1,
-                            stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+                            stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
+                            universal_newlines=True)
 
     q = Queue()
     t = Thread(target=enqueue_output, args=(proc.stdout, q))
@@ -859,7 +897,7 @@ def shell(cmd, msg=None, allow_fail=False, exit_on_fail=True,
     lastdot = time.time()
     interval = 0.002
     max_interval = 0.5
-    while t.isAlive():
+    while t.is_alive():
         now = time.time()
         if now - lastdot > 4 and msg:
             sys.stdout.write('.')
@@ -882,7 +920,7 @@ def shell(cmd, msg=None, allow_fail=False, exit_on_fail=True,
         if not allow_fail and exit_on_fail:
             console('Command failed: %s' % cmd)
             for line in tail:
-                print '  ', line
+                print('  ', line)
             if LOGFILE:
                 console('See log for details: %s' % LOGFILE)
             sys.exit(1)
@@ -896,7 +934,7 @@ def shell(cmd, msg=None, allow_fail=False, exit_on_fail=True,
 
     if save_output:
         if output:
-            return '\n'.join(output)
+            return '\n'.join(str(x) for x in output)
         return ''
     return None
 
@@ -926,15 +964,14 @@ def in_venv():
 
 
 def check_git():
+    """Checks if git installed, raises ShellFailed if not."""
     try:
         shell(cmd='git --version',
               msg='Checking if git is installed',
               allow_fail=True)
         return True
     except ShellFailed:
-        console('no\ngit is not installed, please install git to continue',
-                lvl=logging.ERROR)
-        sys.exit(1)
+        raise ShellFailed('git is not installed')
 
 
 def check_install_pip():
@@ -976,6 +1013,15 @@ def check_virtualenv():
         return False
 
 
+def check_vcpython27():
+    try:
+        shell(cmd='where /R C:\\Users Visual*C++*2008*-bit*Command*Prompt.lnk',
+              allow_fail=True)
+        return True
+    except ShellFailed:
+        return False
+
+
 def run():
     # Main entry point as a script from setup.py
     # If run as a script directly
