@@ -1,4 +1,4 @@
-# Copyright (c) 2014 Riverbed Technology, Inc.
+# Copyright (c) 2019 Riverbed Technology, Inc.
 #
 # This software is licensed under the terms and conditions of the MIT License
 # accompanying the software ("License").  This software is distributed "AS IS"
@@ -7,8 +7,9 @@
 
 import os
 import re
-import logging
 import sys
+import shutil
+import logging
 import tempfile
 
 import steelscript.commands.steel
@@ -26,10 +27,12 @@ logger = logging.getLogger(__name__)
 class TestVirtualEnv(unittest.TestCase):
 
     def setUp(self):
+        self.skipTest('Skip until Python3 is on main branch')
+
         self.venv = tempfile.mkdtemp()
         self.env = {}
         current_venv = None
-        for k, v in os.environ.iteritems():
+        for k, v in os.environ.items():
             if k == 'VIRTUAL_ENV':
                 current_venv = v
                 continue
@@ -47,7 +50,7 @@ class TestVirtualEnv(unittest.TestCase):
             if m:
                 path = ':'.join([e for e in m.groups() if e])
         self.env['PATH'] = path
-        shell('virtualenv {venv}'.format(venv=self.venv),
+        shell('virtualenv -p $(which python) {venv}'.format(venv=self.venv),
               env=self.env, exit_on_fail=False)
 
         self.steel = 'python ' + os.path.abspath(
@@ -65,8 +68,7 @@ class TestVirtualEnv(unittest.TestCase):
                      env=self.env, exit_on_fail=False, save_output=True)
 
     def tearDown(self):
-        #shutil.rmtree(self.venv)
-        pass
+        shutil.rmtree(self.venv)
 
 
 class TestInstallGitlab(TestVirtualEnv):
@@ -76,7 +78,7 @@ class TestInstallGitlab(TestVirtualEnv):
         self.assertFalse('steel' in out)
 
         with self.assertRaises(ShellFailed):
-            out = self.shell('steel -h')
+            out = self.shell('{venv}/bin/steel -h'.format(venv=self.venv))
 
         out = self.shell('vsteel -h')
         self.assertTrue('Usage' in out)
@@ -174,13 +176,13 @@ class TestInstallLocalGit(TestVirtualEnv):
         self.assertFalse('steel' in out)
 
         with self.assertRaises(ShellFailed):
-            out = self.shell('steel -h')
+            out = self.shell('{venv}/bin/steel -h'.format(venv=self.venv))
 
         gitdir = os.path.join(self.venv, 'git')
         self.shell('mkdir {gitdir}'.format(gitdir=gitdir))
 
         relpath = os.path.abspath(__file__)
-        for i in xrange(5):
+        for i in range(5):
             relpath = os.path.dirname(relpath)
 
         pkgs = steelscript.commands.steel.STEELSCRIPT_CORE
@@ -191,7 +193,9 @@ class TestInstallLocalGit(TestVirtualEnv):
             if not os.path.exists(pkgdir):
                 self.skipTest('Test skipped to avoid failure due to alternate '
                               'directory structure.')
-                raise Exception('Cannot find git directory: {0}'.format(pkgdir))
+                raise Exception(
+                    'Cannot find git directory: {0}'.format(pkgdir)
+                )
             os.symlink(pkgdir, os.path.join(gitdir, pkg + '.git'))
 
         out = self.shell('vsteel install --giturl file://{gitdir}'
