@@ -1,12 +1,10 @@
-# Copyright (c) 2014 Riverbed Technology, Inc.
+# Copyright (c) 2019 Riverbed Technology, Inc.
 #
 # This software is licensed under the terms and conditions of the MIT License
 # accompanying the software ("License").  This software is distributed "AS IS"
 # as set forth in the License.
 
 
-
-from __future__ import absolute_import
-
 from steelscript.common.service import *
 from steelscript.common.timeutils import *
+from steelscript.common.interval import *
