@@ -1,4 +1,4 @@
-# Copyright (c) 2014 Riverbed Technology, Inc.
+# Copyright (c) 2019 Riverbed Technology, Inc.
 #
 # This software is licensed under the terms and conditions of the MIT License
 # accompanying the software ("License").  This software is distributed "AS IS"
@@ -8,13 +8,19 @@
 import os
 import ssl
 import json
+import errno
+import urllib.request
+import urllib.parse
+import urllib.error
+import socket
 import logging
 import tempfile
-import urlparse
-from xml.etree import ElementTree
-
 import requests
+import mimetypes
 import requests.exceptions
+import warnings
+from os.path import basename
+from xml.etree import ElementTree
 from requests.utils import default_user_agent
 from requests.adapters import HTTPAdapter
 from requests.structures import CaseInsensitiveDict
@@ -22,12 +28,17 @@ from requests.packages.urllib3.util import parse_url
 from requests.packages.urllib3.poolmanager import PoolManager
 from pkg_resources import get_distribution
 
-from steelscript.common.exceptions import RvbdException, RvbdHTTPException
+from steelscript.common.exceptions import RvbdException, RvbdHTTPException, \
+    RvbdConnectException
 
 logger = logging.getLogger(__name__)
 rest_logger = logging.getLogger('REST')
 
 
+warnings.catch_warnings()
+warnings.simplefilter('once')
+
+
 class SSLAdapter(HTTPAdapter):
     """ An HTTPS Transport Adapter that uses an arbitrary SSL version. """
     # handle https connections that don't like to negotiate
@@ -47,7 +58,7 @@ class SSLAdapter(HTTPAdapter):
 def scrub_passwords(data):
     if hasattr(data, 'iteritems'):
         result = {}
-        for (k, v) in data.iteritems():
+        for (k, v) in data.items():
             if k.lower() in ('password', 'authenticate', 'cookie'):
                 result[k] = "********"
             else:
@@ -62,6 +73,28 @@ def scrub_passwords(data):
         return data
 
 
+def test_tcp_conn(dest, port):
+    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    s.settimeout(5)
+    try:
+        rcode = s.connect_ex((dest, port))
+    except Exception as e:
+        raise RvbdConnectException("Exception raised in test_tcp_conn({2}, "
+                                   "{3}) type: {0}, message {1}"
+                                   "".format(type(e), str(e), dest, port))
+    finally:
+        s.close()
+
+    if rcode == 0:
+        return True
+    else:
+        err_name = errno.errorcode.get(rcode, "UnknownError")
+        raise RvbdConnectException("Socket errno raised on connection "
+                                   "attempt: {0}".format(err_name),
+                                   errno=rcode,
+                                   errname=err_name)
+
+
 class Connection(object):
     """ Handle authentication and communication to remote machines. """
 
@@ -116,6 +149,7 @@ class Connection(object):
         self.conn.verify = verify
         self._reauthenticate_handler = reauthenticate_handler
         self.set_user_agent()
+        self.cookies = None
 
         # store last full response
         self.response = None
@@ -125,9 +159,13 @@ class Connection(object):
     def __repr__(self):
         return '<{0} to {1}>'.format(self.__class__.__name__, self.hostname)
 
+    def __del__(self):
+        # cleanup after ourselves
+        self.conn.close()
+
     def get_url(self, path):
         """ Returns a fully qualified URL given a path. """
-        return urlparse.urljoin(self.hostname, path)
+        return urllib.parse.urljoin(self.hostname, path)
 
     def set_user_agent(self, extra=None):
         version = get_distribution('steelscript').version
@@ -143,21 +181,20 @@ class Connection(object):
 
     def _request(self, method, path, body=None, params=None,
                  extra_headers=None, raw_json=None, stream=False,
-                 **kwargs):
+                 files=None, **kwargs):
         p = parse_url(path)
         if not p.host:
             path = self.get_url(path)
-
         try:
             rest_logger.info('%s %s' % (method, str(path)))
             if params:
                 rest_logger.info('Parameters: ')
-                for k, v in params.iteritems():
+                for k, v in params.items():
                     rest_logger.info('... %s: %s' % (k, v))
 
             if self.REST_DEBUG >= 1 and extra_headers:
                 rest_logger.info('Extra request headers: ')
-                for k, v in extra_headers.iteritems():
+                for k, v in extra_headers.items():
                     rest_logger.info('... %s: %s' % (k, v))
             if self.REST_DEBUG >= 2 and body:
                 rest_logger.info('Request body: ')
@@ -171,7 +208,11 @@ class Connection(object):
                         debug_body = json.dumps(
                             raw_json, indent=2, cls=self.JsonEncoder)
                 else:
-                    debug_body = body
+                    if files and isinstance(body, dict):
+                        debug_body = json.dumps(body, indent=2,
+                                                cls=self.JsonEncoder)
+                    else:
+                        debug_body = body
                 lines = debug_body.split('\n')
                 for line in lines[:self.REST_BODY_LINES]:
                     rest_logger.info('... %s' % line)
@@ -181,14 +222,16 @@ class Connection(object):
 
             r = self.conn.request(method, path, data=body, params=params,
                                   headers=extra_headers,
-                                  stream=stream, **kwargs)
+                                  stream=stream,
+                                  files=files,
+                                  cookies=self.cookies, **kwargs)
 
             if r.request.url != path:
                 rest_logger.info('Full URL: %s' % r.request.url)
 
             if self.REST_DEBUG >= 1 and extra_headers:
                 rest_logger.info('Request headers: ')
-                for k, v in scrub_passwords(r.request.headers).iteritems():
+                for k, v in scrub_passwords(r.request.headers).items():
                     rest_logger.info('... %s: %s' % (k, v))
 
             if stream:
@@ -200,7 +243,7 @@ class Connection(object):
 
             if self.REST_DEBUG >= 1 and extra_headers:
                 rest_logger.info('Response headers: ')
-                for k, v in r.headers.iteritems():
+                for k, v in r.headers.items():
                     rest_logger.info('... %s: %s' % (k, v))
 
             if self.REST_DEBUG >= 2 and not stream and r.text:
@@ -230,25 +273,31 @@ class Connection(object):
             self._ssladapter = True
             logger.info('SSL error -- retrying with TLSv1')
             r = self.conn.request(method, path, data=body,
-                                  params=params, headers=extra_headers)
+                                  params=params, headers=extra_headers,
+                                  cookies=self.cookies, files=files)
 
         # check if good status response otherwise raise exception
         if not r.ok:
             exc = RvbdHTTPException(r, r.text, method, path)
             if (self._reauthenticate_handler is not None and
-                exc.error_id in ('AUTH_INVALID_SESSION',
-                                 'AUTH_EXPIRED_TOKEN')):
+                exc.error_id in ('AUTH_REQUIRED',
+                                 'AUTH_INVALID_SESSION',
+                                 'AUTH_EXPIRED_TOKEN',
+                                 'AUTH_INVALID_CREDENTIALS')):
                 logger.debug('session timed out -- reauthenticating')
+                # clean any stale cookies from session
+                self._clear_cookies()
                 handler = self._reauthenticate_handler
                 self._reauthenticate_handler = None
                 handler()
                 logger.debug('session reauthentication succeeded -- retrying')
-                r = self._request(method, path, body, params,
-                                  extra_headers, raw_json=raw_json, **kwargs)
+                r = self._request(method, path, body=body, params=params,
+                                  extra_headers=extra_headers,
+                                  raw_json=raw_json,
+                                  files=files, **kwargs)
                 # successful connection, reset token if previously unset
                 self._reauthenticate_handler = handler
                 return r
-
             else:
                 raise exc
 
@@ -266,15 +315,22 @@ class Connection(object):
                     res = obj.__dict__
             return res
 
+    def _clear_cookies(self):
+        self.conn.headers.pop('Cookie', None)
+        if self.cookies:
+            self.cookies.clear_session_cookies()
+        self.conn.cookies.clear_session_cookies()
+
     def _prepare_headers(self, headers):
         if headers:
             return CaseInsensitiveDict(headers)
         else:
             return CaseInsensitiveDict()
 
-    def json_request(self, method, path, body=None,
-                     params=None, extra_headers=None, raw_response=False):
+    def json_request(self, method, path, body=None, params=None,
+                     extra_headers=None, raw_response=False):
         """ Send a JSON request and receive JSON response. """
+
         extra_headers = self._prepare_headers(extra_headers)
         extra_headers['Content-Type'] = 'application/json'
         extra_headers['Accept'] = 'application/json'
@@ -289,10 +345,13 @@ class Connection(object):
                           raw_json=raw_json)
 
         if r.status_code == 204 or len(r.content) == 0:
-            return None  # no data
+            data = None  # no data
+        else:
+            data = json.loads(r.text)
+
         if raw_response:
-            return json.loads(r.text), r
-        return json.loads(r.text)
+            return data, r
+        return data
 
     def xml_request(self, method, path, body=None,
                     params=None, extra_headers=None, raw_response=False):
@@ -320,6 +379,134 @@ class Connection(object):
 
         return tree
 
+    def urlencoded_request(self, method, path, body=None, params=None,
+                           extra_headers=None, raw_response=False):
+        """Send a request with url encoded parameters in body"""
+        extra_headers = self._prepare_headers(extra_headers)
+        extra_headers['Content-Type'] = 'application/x-www-form-urlencoded'
+        extra_headers['Accept'] = 'application/json'
+
+        body = urllib.parse.urlencode(body)
+
+        return self._request(method, path, body, params, extra_headers)
+
+    def upload_file(self, path, files, body=None, params=None,
+                    extra_headers=None, file_headers=None, field_name='file',
+                    raw_response=False):
+        """
+        Executes a POST to upload a file or files.
+
+        :param path: The full or relative URL of the file upload API
+        :param files: Can be a string that is the full path to a file to be
+               uploaded OR it can be a tuple/list of strings that are each the
+               full path to a file to be uploaded.
+        :param body: Optional body. If present must be a dictionary.
+        :param params: optional URL params
+        :param extra_headers: Optional headers
+        :param file_headers: Optional headers to include with the multipart
+               file data. Default is {'Expires': '0'}. Pass in an empty dict
+               object if you would not like to include any file_headers in the
+               multipart data.
+        :param field_name: The name of the form field on the destination that
+               will receive the posted multipart data. Default is 'file'
+        :param raw_response: False (defualt) results in the function returning
+               only the decoded JSON response present in the response body. If
+               set to True then the funciton will return a tuple of the decoded
+               JSON body and the full response object. Set to True if you
+               want to inspect the result code or response headers.
+        :return: See 'raw_response' for details on the returned data.
+        """
+
+        # the underlying request object will add the correct content type
+        # header
+        extra_headers = self._prepare_headers(extra_headers)
+        extra_headers['Accept'] = 'application/json'
+
+        # by default add a zero expiration header.
+        if file_headers is None:
+            file_headers = {'Expires': '0'}
+
+        # body must be a dict object for this call.
+        if (body is not None) and (not isinstance(body, dict)):
+            raise RvbdException("The 'body' argument must either be None or a "
+                                "dict")
+        # Open all of the files
+        xfiles = dict()
+        if isinstance(files, str):
+            try:
+                xfiles[basename(files)] = {'file': open(files, 'rb')}
+            except IOError:
+                raise RvbdException("Could not open '{0}' for read in binary "
+                                    "mode. Please check path.".format(files))
+        elif isinstance(files, (list, tuple)):
+            for file in files:
+                try:
+                    xfiles[basename(file)] = {'file': open(file, 'rb')}
+                except IOError:
+                    raise RvbdException("Could not open '{0}' for read in "
+                                        "binary mode. Please check path."
+                                        "".format(file))
+        else:
+            raise RvbdException("upload_file 'files' argument must be a "
+                                "string or list type (list, tuple). {0} is "
+                                "not a valid files argument."
+                                "".format(type(files)))
+
+        # build the multipart content from the files
+        if len(xfiles) == 1:
+            # single file is a dict object
+            for f in xfiles:
+                mtype, _ = mimetypes.guess_type(f)
+                if mtype and list(file_headers.keys()):
+                    req_files = {field_name: (f,
+                                              xfiles[f]['file'],
+                                              mtype,
+                                              file_headers)}
+                elif mtype and not list(file_headers.keys()):
+                    req_files = {field_name: (f,
+                                              xfiles[f]['file'],
+                                              mtype)}
+                else:
+                    req_files = {field_name: (f,
+                                              xfiles[f]['file'])}
+
+        elif len(xfiles) > 1:
+            # multiple files is a list
+            req_files = list()
+            for f in xfiles:
+                mtype, _ = mimetypes.guess_type(f)
+                if mtype and list(file_headers.keys()):
+                    req_files.append((field_name, (f,
+                                                   xfiles[f]['file'],
+                                                   mtype,
+                                                   file_headers)
+                                      ))
+                elif mtype and not list(file_headers.keys()):
+                    req_files.append((field_name, (f,
+                                                   xfiles[f]['file'],
+                                                   mtype)
+                                      ))
+                else:
+                    req_files.append((field_name, (f,
+                                                   xfiles[f]['file'])
+                                      ))
+        else:
+            raise RvbdException("At least one valid file required. Files was: "
+                                "{0}".format(files))
+
+        # send the files
+        r = self._request("POST", path, body, params, extra_headers,
+                          files=req_files)
+
+        if r.status_code == 204 or len(r.content) == 0:
+            data = None  # no data
+        else:
+            data = json.loads(r.text)
+
+        if raw_response:
+            return data, r
+        return data
+
     def upload(self, path, data, method="POST", params=None,
                extra_headers=None):
         """Upload raw data to the given URL path with the given content type.
