@@ -1,4 +1,4 @@
-# Copyright (c) 2014 Riverbed Technology, Inc.
+# Copyright (c) 2019 Riverbed Technology, Inc.
 #
 # This software is licensed under the terms and conditions of the MIT License
 # accompanying the software ("License").  This software is distributed "AS IS"
@@ -21,10 +21,11 @@ the Service class.  A script that interacts with both namespaces must
 instantiate two separate objects.
 """
 
-from __future__ import absolute_import
 
 import base64
 import logging
+import hashlib
+import time
 
 from steelscript.common import connection
 from steelscript.common.exceptions import RvbdException, RvbdHTTPException
@@ -37,6 +38,7 @@ logger = logging.getLogger(__name__)
 
 
 class Auth(object):
+    NONE = 0
     BASIC = 1
     COOKIE = 2
     OAUTH = 3
@@ -141,9 +143,11 @@ class Service(object):
         if self.conn is not None and hasattr(self.conn, 'close'):
             self.conn.close()
 
-        self.conn = connection.Connection(self.host, port=self.port,
-                                          verify=self.verify_ssl,
-                                          reauthenticate_handler=self.reauthenticate)
+        self.conn = connection.Connection(
+            self.host, port=self.port,
+            verify=self.verify_ssl,
+            reauthenticate_handler=self.reauthenticate
+        )
 
     def logout(self):
         """End the authenticated session with the device."""
@@ -176,9 +180,10 @@ class Service(object):
                              (self.service, v))
                 return True
 
-        raise RvbdException("API version(s) %s not supported (supported version(s): %s)" %
-                           (', '.join([str(v) for v in api_versions]),
-                            ', '.join([str(v) for v in self.supported_versions])))
+        msg = ("API version(s) %s not supported (supported version(s): %s)" %
+               (', '.join([str(v) for v in api_versions]),
+                ', '.join([str(v) for v in self.supported_versions])))
+        raise RvbdException(msg)
 
     def _get_supported_versions(self):
         """Get the common list of services and versions supported."""
@@ -199,11 +204,12 @@ class Service(object):
 
         try:
             auth_info = self.conn.json_request('GET', path)
+            supported_methods = auth_info['supported_methods']
             logger.info("Supported authentication methods: %s" %
-                        (','.join(auth_info['supported_methods'])))
-            self._supports_auth_basic = ("BASIC" in auth_info['supported_methods'])
-            self._supports_auth_cookie = ("COOKIE" in auth_info['supported_methods'])
-            self._supports_auth_oauth = ("OAUTH_2_0" in auth_info['supported_methods'])
+                        (','.join(supported_methods)))
+            self._supports_auth_basic = ("BASIC" in supported_methods)
+            self._supports_auth_cookie = ("COOKIE" in supported_methods)
+            self._supports_auth_oauth = ("OAUTH_2_0" in supported_methods)
         except RvbdHTTPException as e:
             if e.status != 404:
                 raise
@@ -225,22 +231,32 @@ class Service(object):
         self._detect_auth_methods()
 
         if self._supports_auth_oauth and Auth.OAUTH in self.auth.methods:
-            # TODO fix for future support to handle appropriate triplets
-            code = self.auth.access_code
             path = '/api/common/1.0/oauth/token'
-            data = {
-                'grant_type': 'access_code',
-                'assertion': code
-            }
-            answer = self.conn.json_request('POST', path, 'POST', params=data)
-            token = answer['access_token']
+            assertion = '.'.join([
+                base64.urlsafe_b64encode(b'{"alg":"none"}').decode(),
+                self.auth.access_code,
+                ''
+            ])
+            state = hashlib.md5(str(time.time()).encode()).hexdigest()
+            data = {'grant_type': 'access_code',
+                    'assertion': assertion,
+                    'state': state}
+            answer = self.conn.urlencoded_request('POST', path,
+                                                  body=data)
+
+            if answer.json()['state'] != state:
+                msg = "Inconsistent state value in OAuth response"
+                raise RvbdException(msg)
+
+            token = answer.json()['access_token']
             st = token.split('.')
             if len(st) == 1:
                 auth_header = 'Bearer %s' % token
             elif len(st) == 3:
                 auth_header = 'SignedBearer %s' % token
             else:
-                raise RvbdException('Unknown OAuth response from server: %s' % st)
+                msg = 'Unknown OAuth response from server: %s' % st
+                raise RvbdException(msg)
             self.conn.add_headers({'Authorization': auth_header})
             logger.info('Authenticated using OAUTH2.0')
 
@@ -251,20 +267,21 @@ class Service(object):
                 "password": self.auth.password
             }
 
-            response_body, http_response = self.conn.json_request('POST', path, body=data,
-                                                                  raw_response=True)
+            body, http_response = self.conn.json_request('POST', path,
+                                                         body=data,
+                                                         raw_response=True)
 
             # we're good, set up our http headers for subsequent
             # requests!
-            cookie = http_response.headers['set-cookie']
-            self.conn.add_headers({'Cookie': cookie})
+            self.conn.cookies = http_response.cookies
 
             logger.info("Authenticated using COOKIE")
 
         elif self._supports_auth_basic and Auth.BASIC in self.auth.methods:
 
             # Use HTTP Basic authentication
-            s = base64.b64encode("%s:%s" % (self.auth.username, self.auth.password))
+            s = base64.b64encode("%s:%s" % (self.auth.username,
+                                            self.auth.password))
             self.conn.add_headers({'Authorization': 'Basic %s' % s})
 
             logger.info("Authenticated using BASIC")
