@@ -1,22 +1,20 @@
-# Copyright (c) 2014 Riverbed Technology, Inc.
+# Copyright (c) 2019 Riverbed Technology, Inc.
 #
 # This software is licensed under the terms and conditions of the MIT License
 # accompanying the software ("License").  This software is distributed "AS IS"
 # as set forth in the License.
 
-
 """
 This module contains a number of utilities for working with
 dates and times, in conjunction with the python `datetime` module.
 """
 
-from __future__ import absolute_import
-
 import re
 import time
 import calendar
 from datetime import datetime, timedelta, tzinfo
 from decimal import Decimal
+from dateutil.relativedelta import relativedelta
 
 __all__ = ['ensure_timezone', 'force_to_utc', 'datetime_to_seconds',
            'datetime_to_microseconds', 'datetime_to_nanoseconds',
@@ -265,6 +263,61 @@ def timedelta_total_seconds(td):
     return float(td.microseconds + (td.seconds + td.days * 24 * 3600) * 10**6) / 10**6
 
 
+# move out of TimeParser scope since Python3 won't allow it
+# ref: https://stackoverflow.com/a/13913933/2157429
+class _informat(object):
+    def __init__(self, pattern, has_date, has_year):
+        self.pattern = pattern
+        self.has_date = has_date
+        self.has_year = has_year
+        if has_year:
+            assert has_date
+
+    def match(self, s):
+        tm = datetime.strptime(s, self.pattern)
+
+        if self.has_year:
+            return tm
+
+        now = datetime.now()
+        if self.has_date:
+            return tm.replace(year=now.year)
+
+        return tm.replace(month=now.month, day=now.day, year=now.year)
+
+
+# all the different time of day formats we can parse.
+# refer to the python time module documentation for details
+# on the format strings
+_time_formats = (
+    '%H:%M:%S.%f',
+    '%H:%M:%S',
+    '%H:%M',
+    '%I:%M:%S %p',
+    '%I:%M:%S%p',
+    '%I:%M %p',
+    '%I:%M%p',
+    '%I %p',
+    '%I%p'
+)
+
+# all the different date formats we can parse.
+# note that dates are "normalized" a bit, see parse() below.
+_date_formats = (
+    '%m/%d',
+    '%B %d',
+    '%b %d'
+)
+
+_date_year_formats = (
+    '%Y/%m/%d',
+    '%m/%d/%Y',
+    '%m/%d/%y',
+    '%B/%d/%Y',
+    '%b/%d/%Y',
+)
+
+
 class TimeParser(object):
     """Instances of this class parse strings representing dates and/or
     times into python `datetime.datetime` objects.
@@ -325,66 +378,21 @@ class TimeParser(object):
         dt, self._fmt = self._parse_no_hint(s)
         return dt
 
-    # all the different time of day formats we can parse.
-    # refer to the python time module documentation for details
-    # on the format strings
-    _time_formats = (
-        '%H:%M:%S.%f',
-        '%H:%M:%S',
-        '%H:%M',
-        '%I:%M:%S %p',
-        '%I:%M:%S%p',
-        '%I:%M %p',
-        '%I:%M%p',
-        '%I %p',
-        '%I%p'
-    )
 
-    # all the different date formats we can parse.
-    # note that dates are "normalized" a bit, see parse() below.
-    _date_formats = (
-        '%m/%d',
-        '%B %d',
-        '%b %d'
-    )
+    _formats = (
+        (
+            [_informat(_tf, False, False) for _tf in _time_formats]
+            + [_informat('%s %s' % (_tf, _df), True, False)
+               for _tf in _time_formats for _df in _date_formats]
+            + [_informat('%s %s' % (_df, _tf), True, False)
+               for _tf in _time_formats for _df in _date_formats]
+            + [_informat('%s %s' % (_tf, _df), True, True)
+               for _tf in _time_formats for _df in _date_year_formats]
+            + [_informat('%s %s' % (_df, _tf), True, True)
+               for _tf in _time_formats for _df in _date_year_formats]
+        )
+     )
 
-    _date_year_formats = (
-        '%Y/%m/%d',
-        '%m/%d/%Y',
-        '%m/%d/%y',
-        '%B/%d/%Y',
-        '%b/%d/%Y',
-    )
-
-    class _informat(object):
-        def __init__(self, pattern, has_date, has_year):
-            self.pattern = pattern
-            self.has_date = has_date
-            self.has_year = has_year
-            if has_year:
-                assert has_date
-
-        def match(self, s):
-            tm = datetime.strptime(s, self.pattern)
-
-            if self.has_year:
-                return tm
-
-            now = datetime.now()
-            if self.has_date:
-                return tm.replace(year=now.year)
-
-            return tm.replace(month=now.month, day=now.day, year=now.year)
-
-    _formats = ([_informat(_tf, False, False) for _tf in _time_formats]
-                + [_informat('%s %s' % (_tf, _df), True, False)
-                   for _tf in _time_formats for _df in _date_formats]
-                + [_informat('%s %s' % (_df, _tf), True, False)
-                   for _tf in _time_formats for _df in _date_formats]
-                + [_informat('%s %s' % (_tf, _df), True, True)
-                   for _tf in _time_formats for _df in _date_year_formats]
-                + [_informat('%s %s' % (_df, _tf), True, True)
-                   for _tf in _time_formats for _df in _date_year_formats])
 
 _timedelta_units = {
     'us': 0.000001, 'usec': 0.000001, 'microsecond': 0.000001, 'microseconds': 0.000001,
@@ -394,9 +402,32 @@ _timedelta_units = {
     'h': 60*60, 'hr': 60*60, 'hour': 60*60, 'hours': 60*60,
     'd': 24*60*60, 'day': 24*60*60, 'days': 24*60*60,
     'w': 7*24*60*60, 'week': 7*24*60*60, 'weeks': 7*24*60*60,
+    'month': 30*24*60*60, 'months': 30*24*60*60,
+    'q': 91*24*60*60, 'quarter': 91*24*60*60, 'quarters': 91*24*60*60,
+    'y': 365*24*60*60, 'year': 365*24*60*60, 'years': 365*24*60*60
 }
 
-_timedelta_re = re.compile("([0-9]*\.*[0-9]*) *([a-zA-Z]*)")
+_timedelta_re = re.compile("([0-9]*\\.*[0-9]*) *([a-zA-Z]*)")
+
+# Build map from casual units to standard units recognizable by datetime
+units_map = {}
+for k, v in _timedelta_units.items():
+    if v == 1:
+        units_map[k] = 'second'
+    elif v == 60:
+        units_map[k] = 'minute'
+    elif v == 60*60:
+        units_map[k] = 'hour'
+    elif v == 60*60*24:
+        units_map[k] = 'day'
+    elif v == 60*60*24*7:
+        units_map[k] = 'week'
+    elif v == 60*60*24*30:
+        units_map[k] = 'month'
+    elif v == 60*60*24*91:
+        units_map[k] = 'quarter'
+    elif v == 60*60*24*365:
+        units_map[k] = 'year'
 
 
 def timedelta_str(td):
@@ -490,7 +521,45 @@ def parse_timedelta(s):
     return timedelta(seconds=units * float(val))
 
 
-def parse_range(s):
+def floor_dt(dt, unit, begin_monday=False):
+    """Derive the most recent start datetime of the current duration unit.
+    i.e., for datetime(2016, 12, 14, 10, 14) and hour, should return
+    datetime(2016, 12, 14, 10, 0)
+
+    :param dt: datetime value
+    :param unit: string, duration unit: year, quarter, month, week, day,
+        hour, minute, second.
+    :return : datetime.
+    """
+
+    if unit == 'week':
+        offset = 0 if begin_monday else 1
+        dt = dt - timedelta(days=dt.weekday() + offset)
+        return floor_dt(dt, 'day')
+
+    if unit == 'quarter':
+        # Update the month to be first of current quarter
+        dt = dt.replace(month=(dt.month - 1)//3 * 3 + 1)
+        return floor_dt(dt, 'month')
+
+    start_month, start_day, start_hour, start_minute, start_second = \
+        1, 1, 0, 0, 0
+
+    kwargs, smaller_unit = {}, False
+
+    for u in ['year', 'month', 'day', 'hour', 'minute', 'second']:
+        if smaller_unit:
+            kwargs[u] = eval('start_' + u)
+        else:
+            kwargs[u] = getattr(dt, u)
+
+        if not smaller_unit and u == unit:
+            smaller_unit = True
+
+    return datetime(**kwargs)
+
+
+def parse_range(s, begin_monday=False):
     """Parse the string `s` representing a range of times
     (e.g., `"12:00 PM to 1:00 PM"` or `"last 2 weeks"`).
 
@@ -501,6 +570,11 @@ def parse_range(s):
 
     s = s.strip()
 
+    if s == 'today':
+        s = 'this day'
+    elif s == 'yesterday':
+        s = 'previous day'
+
     # first try something of the form "time1 to time2"
     i = s.split('to')
     if len(i) == 2:
@@ -522,6 +596,42 @@ def parse_range(s):
         except ValueError:
             pass
 
+    # try to convert the datetime to
+    elif s.startswith('previous'):
+        try:
+            duration = s[8:].strip()
+            unit = units_map[_timedelta_re.match(duration).group(2)]
+            now = datetime.now()
+
+            how_many = _timedelta_re.match(duration).group(1)
+            how_many = 1 if how_many == '' else int(how_many)
+
+            if unit == 'quarter':
+                delta = relativedelta(months=how_many*3)
+            else:
+                delta = relativedelta(**{unit+'s': how_many})
+
+            start = floor_dt(now - delta, unit, begin_monday)
+            end = floor_dt(now, unit, begin_monday)
+
+            return start, end
+
+        except ValueError:
+            pass
+
+    elif s.startswith('this'):
+        try:
+            duration = s[4:].strip()
+            unit = units_map[_timedelta_re.match(duration).group(2)]
+            now = datetime.now()
+
+            start = floor_dt(now, unit, begin_monday)
+
+            return start, now
+
+        except ValueError:
+            pass
+
     # if it still doesn't work, try the pilot time filter syntax
     # which looks like "time1,time2"
     i = s.split(',')
